import { ethers } from "ethers";
import { BuildOfferResponse, Offer, ListNFTsResponse, GetNFTResponse, ListCollectionOffersResponse, GetAssetsResponse, GetOrdersResponse, GetPaymentTokensResponse, GetBundlesResponse } from "./types";
import { FulfillmentDataResponse, OrderAPIOptions, OrderSide, OrdersQueryOptions, OrderV2, ProtocolData } from "../orders/types";
import { Chain, OpenSeaAPIConfig, OpenSeaAsset, OpenSeaAssetBundle, OpenSeaAssetBundleQuery, OpenSeaAssetQuery, OpenSeaCollection, OpenSeaFungibleTokenQuery } from "../types";
/**
 * The API class for the OpenSea SDK.
 * @category Main Classes
 */
export declare class OpenSeaAPI {
    /**
     * Base url for the API
     */
    readonly apiBaseUrl: string;
    /**
     * Default size to use for fetching orders
     */
    pageSize: number;
    /**
     * Logger function to use when debugging
     */
    logger: (arg: string) => void;
    private apiKey;
    private chain;
    private retryDelay;
    /**
     * Create an instance of the OpenSea API
     * @param config OpenSeaAPIConfig for setting up the API, including an optional API key, Chain name, and base URL
     * @param logger Optional function for logging debug strings before and after requests are made
     */
    constructor(config: OpenSeaAPIConfig, logger?: (arg: string) => void);
    /**
     * Gets an order from API based on query options.
     * @param options
     * @param options.side The side of the order (buy or sell
     * @param options.protocol The protocol, typically seaport, to query orders for
     * @param options.orderDirection The direction to sort the orders
     * @param options.orderBy The field to sort the orders by
     * @param options.limit The number of orders to retrieve
     * @param options.makerAddress Fitler by the wallet addres of the order maker
     * @param options.takerAddress Filter by  wallet address of the order taker
     * @param options.asset_contract_address Address of the NFT's contract
     * @param options.token_ids String array of token IDs to filter by.
     * @param options.listed_after Filter by orders listed after the Unix epoch timestamp in seconds
     * @param options.listed_before Filter by orders listed before the Unix epoch timestamp in seconds
     * @returns The first {@link OrderV2} returned by the API
     *
     * @throws An error if there are no matching orders.
     */
    getOrder({ side, protocol, orderDirection, orderBy, ...restOptions }: Omit<OrdersQueryOptions, "limit">): Promise<OrderV2>;
    /**
     * Gets a list of orders from API based on query options.
     * @param options
     * @param options.side The side of the order (buy or sell)
     * @param options.protocol The protocol, typically seaport, to query orders for
     * @param options.orderDirection The direction to sort the orders
     * @param options.orderBy The field to sort the orders by
     * @param options.limit The number of orders to retrieve
     * @param options.makerAddress Fitler by the wallet addres of the order maker
     * @param options.takerAddress Filter by  wallet address of the order taker
     * @param options.asset_contract_address Address of the NFT's contract
     * @param options.token_ids String array of token IDs to filter by.
     * @param options.listed_after Filter by orders listed after the Unix epoch timestamp in seconds
     * @param options.listed_before Filter by orders listed before the Unix epoch timestamp in seconds
     * @returns The {@link GetOrdersResponse} returned by the API.
     */
    getOrders({ side, protocol, orderDirection, orderBy, ...restOptions }: Omit<OrdersQueryOptions, "limit">): Promise<GetOrdersResponse>;
    /**
     * Generate the data needed to fulfill a listing or an offer onchain.
     * @param fulfillerAddress The wallet address which will be used to fulfill the order
     * @param orderHash The hash of the order to fulfill
     * @param protocolAddress The address of the seaport contract
     * @side The side of the order (buy or sell)
     * @returns The {@link FulfillmentDataResponse}
     */
    generateFulfillmentData(fulfillerAddress: string, orderHash: string, protocolAddress: string, side: OrderSide): Promise<FulfillmentDataResponse>;
    /**
     * Post an order to OpenSea.
     * @param order The order to post
     * @param apiOptions
     * @param apiOptions.protocol The protocol, typically seaport, to post the order to.
     * @param apiOptions.side The side of the order (buy or sell).
     * @param apiOptions.protocolAddress The address of the seaport contract.
     * @param options
     * @param options.retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link OrderV2} posted to the API.
     */
    postOrder(order: ProtocolData, apiOptions: OrderAPIOptions, { retries }?: {
        retries?: number;
    }): Promise<OrderV2>;
    /**
     * Build a OpenSea collection offer.
     * @param offererAddress The wallet address which is creating the offer.
     * @param quantity The number of NFTs requested in the offer.
     * @param collectionSlug The slug (identifier) of the collection to build the offer for.
     * @returns The {@link BuildOfferResponse} returned by the API.
     */
    buildOffer(offererAddress: string, quantity: number, collectionSlug: string): Promise<BuildOfferResponse>;
    /**
     * Get a list collection offers for a given slug.
     * @param slug The slug (identifier) of the collection to list offers for
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link ListCollectionOffersResponse} returned by the API.
     */
    getCollectionOffers(slug: string, retries?: number): Promise<ListCollectionOffersResponse | null>;
    /**
     * Post a collection offer to OpenSea.
     * @param order The collection offer to post.
     * @param slug The slug (identifier) of the collection to post the offer for.
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link Offer} returned to the API.
     */
    postCollectionOffer(order: ProtocolData, slug: string, retries?: number): Promise<Offer | null>;
    /**
     * Fetch an asset.
     * @deprecated Use {@link getNFT} for multichain capabilities.
     * @param options
     * @param options.tokenAddress The asset's contract address.
     * @param options.tokenId The asset's token ID, or null if ERC-20
     * @param retries Number of times to retry if the service is unavailable for any reason
     * @returns The {@link OpenSeaAsset} returned by the API.
     * @throws An error if the function is called on an unsupported chain.
     */
    getAsset({ tokenAddress, tokenId, }: {
        tokenAddress: string;
        tokenId: string | number | null;
    }, retries?: number): Promise<OpenSeaAsset>;
    /**
     * Fetch multiple NFTs for a collection.
     * @param slug The slug (identifier) of the collection
     * @param limit The number of NFTs to retrieve. Must be greater than 0 and less than 51.
     * @param next Cursor to retrieve the next page of NFTs
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link ListNFTsResponse} returned by the API.
     */
    getNFTsByCollection(slug: string, limit?: number | undefined, next?: string | undefined, retries?: number): Promise<ListNFTsResponse>;
    /**
     * Fetch multiple NFTs for a contract.
     * @param chain The NFT's chain.
     * @param address The NFT's contract address.
     * @param limit The number of NFTs to retrieve. Must be greater than 0 and less than 51.
     * @param next Cursor to retrieve the next page of NFTs.
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link ListNFTsResponse} returned by the API.
     */
    getNFTsByContract(chain: Chain, address: string, limit?: number | undefined, next?: string | undefined, retries?: number): Promise<ListNFTsResponse>;
    /**
     * Fetch NFTs owned by an account.
     * @param address The address of the account
     * @param limit The number of NFTs to retrieve. Must be greater than 0 and less than 51.
     * @param next Cursor to retrieve the next page of NFTs
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @param chain The chain to query. Defaults to the chain set in the constructor.
     * @returns The {@link ListNFTsResponse} returned by the API.
     */
    getNFTsByAccount(address: string, limit?: number | undefined, next?: string | undefined, retries?: number, chain?: Chain): Promise<ListNFTsResponse>;
    /**
     * Fetch metadata, traits, ownership information, and rarity for a single NFT.
     * @param chain The NFT's chain.
     * @param address The NFT's contract address.
     * @param identifier the identifier of the NFT (i.e. Token ID)
     * @param retries Number of times to retry if the service is unavailable for any reason
     * @returns The {@link GetNFTResponse} returned by the API.
     */
    getNFT(chain: Chain, address: string, identifier: string, retries?: number): Promise<GetNFTResponse>;
    /**
     * Fetch a list of assets.
     * @deprecated Use {@link getNFTsByContract} or {@link getNFTsByCollection} for multichain capabilities.
     * @param query Options to filter the list returned.
     * @param query.owner The wallet address of the owner of the assets.
     * @param query.asset_contract_address The Asset's contract address.
     * @param query.token_ids String array of token IDs to filter by.
     * @param query.order_by The field to order the list by.
     * @param query.order_direction The direction to order the list.
     * @param query.limit The number of assets to retrieve. Must be greater than 0 and less than 201.
     * @param query.cursor Cursor to retrieve the next page of assets.
     * @throws An error if the function is called on an unsupported chain.
     * @returns The {@link GetAssetsResponse} returned by the API.
     */
    getAssets(query?: OpenSeaAssetQuery): Promise<GetAssetsResponse>;
    /**
     * Fetch an OpenSea collection.
     * @param slug The slug (identifier) of the collection.
     * @returns The {@link OpenSeaCollection} returned by the API.
     */
    getCollection(slug: string): Promise<OpenSeaCollection>;
    /**
     * Fetch list of fungible tokens.
     * @param query Query to use for getting tokens. See {@link OpenSeaFungibleTokenQuery}.
     * @param page Page number to fetch. Defaults to 1.
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @throws An error if the function is called on an unsupported chain.
     * @returns The {@link GetPaymentTokensResponse} returned by the API.
     */
    getPaymentTokens(query?: OpenSeaFungibleTokenQuery, page?: number, retries?: number): Promise<GetPaymentTokensResponse>;
    /**
     * Fetch a bundle from the API.
     * @param options
     * @param options.slug The bundle's identifier
     * @returns The {@link OpenSeaAssetBundle} returned by the API. If not found, returns null.
     */
    getBundle({ slug, }: {
        slug: string;
    }): Promise<OpenSeaAssetBundle | null>;
    /**
     * Fetch list of bundles from the API.
     * @param query Query to use for getting bunldes. See {@link OpenSeaAssetBundleQuery}.
     * @param page Page number to fetch. Defaults to 1.
     * @returns The {@link GetBundlesResponse} returned by the API.
     */
    getBundles(query?: OpenSeaAssetBundleQuery, page?: number): Promise<GetBundlesResponse>;
    /**
     * Force refresh the metadata for an NFT.
     * @param chain The chain where the NFT is located.
     * @param address The address of the NFT's contract.
     * @param identifier The identifier of the NFT.
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The response from the API.
     */
    refreshNFTMetadata(chain: Chain, address: string, identifier: string, retries?: number): Promise<Response>;
    /**
     * Generic fetch method for any API endpoint
     * @param apiPath Path to URL endpoint under API
     * @param query URL query params. Will be used to create a URLSearchParams object.
     * @returns @typeParam T The response from the API.
     */
    get<T>(apiPath: string, query?: object): Promise<T>;
    /**
     * Generic post methd for any API endpoint.
     * @param apiPath Path to URL endpoint under API
     * @param body Data to send.
     * @param opts ethers ConnectionInfo, similar to Fetch API.
     * @returns @typeParam T The response from the API.
     */
    post<T>(apiPath: string, body?: object, opts?: ethers.utils.ConnectionInfo): Promise<T>;
    private objectToSearchParams;
    /**
     * Get from an API Endpoint, sending auth token in headers
     * @param opts ethers ConnectionInfo, similar to Fetch API
     * @param body Optional body to send. If set, will POST, otherwise GET
     */
    private _fetch;
}
