"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenSeaAPI = void 0;
const ethers_1 = require("ethers");
const constants_1 = require("../constants");
const utils_1 = require("../orders/utils");
const types_1 = require("../types");
const utils_2 = require("../utils/utils");
/**
 * The API class for the OpenSea SDK.
 * @category Main Classes
 */
class OpenSeaAPI {
    /**
     * Create an instance of the OpenSea API
     * @param config OpenSeaAPIConfig for setting up the API, including an optional API key, Chain name, and base URL
     * @param logger Optional function for logging debug strings before and after requests are made
     */
    constructor(config, logger) {
        var _a;
        /**
         * Default size to use for fetching orders
         */
        this.pageSize = 20;
        this.retryDelay = 3000;
        this.apiKey = config.apiKey;
        this.chain = (_a = config.chain) !== null && _a !== void 0 ? _a : types_1.Chain.Mainnet;
        this.apiBaseUrl = (0, utils_2.isTestChain)(this.chain)
            ? constants_1.API_BASE_TESTNET
            : constants_1.API_BASE_MAINNET;
        // Debugging: default to nothing
        this.logger = logger !== null && logger !== void 0 ? logger : ((arg) => arg);
    }
    /**
     * Gets an order from API based on query options.
     * @param options
     * @param options.side The side of the order (buy or sell
     * @param options.protocol The protocol, typically seaport, to query orders for
     * @param options.orderDirection The direction to sort the orders
     * @param options.orderBy The field to sort the orders by
     * @param options.limit The number of orders to retrieve
     * @param options.makerAddress Fitler by the wallet addres of the order maker
     * @param options.takerAddress Filter by  wallet address of the order taker
     * @param options.asset_contract_address Address of the NFT's contract
     * @param options.token_ids String array of token IDs to filter by.
     * @param options.listed_after Filter by orders listed after the Unix epoch timestamp in seconds
     * @param options.listed_before Filter by orders listed before the Unix epoch timestamp in seconds
     * @returns The first {@link OrderV2} returned by the API
     *
     * @throws An error if there are no matching orders.
     */
    getOrder(_a) {
        var { side, protocol = "seaport", orderDirection = "desc", orderBy = "created_date" } = _a, restOptions = __rest(_a, ["side", "protocol", "orderDirection", "orderBy"]);
        return __awaiter(this, void 0, void 0, function* () {
            const { orders } = yield this.get((0, utils_1.getOrdersAPIPath)(this.chain, protocol, side), (0, utils_1.serializeOrdersQueryOptions)(Object.assign({ limit: 1, orderBy,
                orderDirection }, restOptions)));
            if (orders.length === 0) {
                throw new Error("Not found: no matching order found");
            }
            return (0, utils_1.deserializeOrder)(orders[0]);
        });
    }
    /**
     * Gets a list of orders from API based on query options.
     * @param options
     * @param options.side The side of the order (buy or sell)
     * @param options.protocol The protocol, typically seaport, to query orders for
     * @param options.orderDirection The direction to sort the orders
     * @param options.orderBy The field to sort the orders by
     * @param options.limit The number of orders to retrieve
     * @param options.makerAddress Fitler by the wallet addres of the order maker
     * @param options.takerAddress Filter by  wallet address of the order taker
     * @param options.asset_contract_address Address of the NFT's contract
     * @param options.token_ids String array of token IDs to filter by.
     * @param options.listed_after Filter by orders listed after the Unix epoch timestamp in seconds
     * @param options.listed_before Filter by orders listed before the Unix epoch timestamp in seconds
     * @returns The {@link GetOrdersResponse} returned by the API.
     */
    getOrders(_a) {
        var { side, protocol = "seaport", orderDirection = "desc", orderBy = "created_date" } = _a, restOptions = __rest(_a, ["side", "protocol", "orderDirection", "orderBy"]);
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.get((0, utils_1.getOrdersAPIPath)(this.chain, protocol, side), (0, utils_1.serializeOrdersQueryOptions)(Object.assign({ limit: this.pageSize, orderBy,
                orderDirection }, restOptions)));
            return Object.assign(Object.assign({}, response), { orders: response.orders.map(utils_1.deserializeOrder) });
        });
    }
    /**
     * Generate the data needed to fulfill a listing or an offer onchain.
     * @param fulfillerAddress The wallet address which will be used to fulfill the order
     * @param orderHash The hash of the order to fulfill
     * @param protocolAddress The address of the seaport contract
     * @side The side of the order (buy or sell)
     * @returns The {@link FulfillmentDataResponse}
     */
    generateFulfillmentData(fulfillerAddress, orderHash, protocolAddress, side) {
        return __awaiter(this, void 0, void 0, function* () {
            let payload = null;
            if (side === "ask") {
                payload = (0, utils_1.getFulfillListingPayload)(fulfillerAddress, orderHash, protocolAddress, this.chain);
            }
            else {
                payload = (0, utils_1.getFulfillOfferPayload)(fulfillerAddress, orderHash, protocolAddress, this.chain);
            }
            const response = yield this.post((0, utils_1.getFulfillmentDataPath)(side), payload);
            return response;
        });
    }
    /**
     * Post an order to OpenSea.
     * @param order The order to post
     * @param apiOptions
     * @param apiOptions.protocol The protocol, typically seaport, to post the order to.
     * @param apiOptions.side The side of the order (buy or sell).
     * @param apiOptions.protocolAddress The address of the seaport contract.
     * @param options
     * @param options.retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link OrderV2} posted to the API.
     */
    postOrder(order, apiOptions, { retries = 2 } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            let response;
            // TODO: Validate apiOptions. Avoid API calls that will definitely fail
            const { protocol = "seaport", side, protocolAddress } = apiOptions;
            try {
                response = yield this.post((0, utils_1.getOrdersAPIPath)(this.chain, protocol, side), Object.assign(Object.assign({}, order), { protocol_address: protocolAddress }));
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(this.retryDelay);
                return this.postOrder(order, apiOptions, { retries: retries - 1 });
            }
            return (0, utils_1.deserializeOrder)(response.order);
        });
    }
    /**
     * Build a OpenSea collection offer.
     * @param offererAddress The wallet address which is creating the offer.
     * @param quantity The number of NFTs requested in the offer.
     * @param collectionSlug The slug (identifier) of the collection to build the offer for.
     * @returns The {@link BuildOfferResponse} returned by the API.
     */
    buildOffer(offererAddress, quantity, collectionSlug) {
        return __awaiter(this, void 0, void 0, function* () {
            const payload = (0, utils_1.getBuildCollectionOfferPayload)(offererAddress, quantity, collectionSlug);
            const response = yield this.post((0, utils_1.getBuildOfferPath)(), payload);
            return response;
        });
    }
    /**
     * Get a list collection offers for a given slug.
     * @param slug The slug (identifier) of the collection to list offers for
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link ListCollectionOffersResponse} returned by the API.
     */
    getCollectionOffers(slug, retries = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.get((0, utils_1.getCollectionOffersPath)(slug));
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(1000);
                return this.getCollectionOffers(slug, retries - 1);
            }
        });
    }
    /**
     * Post a collection offer to OpenSea.
     * @param order The collection offer to post.
     * @param slug The slug (identifier) of the collection to post the offer for.
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link Offer} returned to the API.
     */
    postCollectionOffer(order, slug, retries = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const payload = (0, utils_1.getPostCollectionOfferPayload)(slug, order);
            try {
                return yield this.post((0, utils_1.getPostCollectionOfferPath)(), payload);
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(1000);
                return this.postCollectionOffer(order, slug, retries - 1);
            }
        });
    }
    /**
     * Fetch an asset.
     * @deprecated Use {@link getNFT} for multichain capabilities.
     * @param options
     * @param options.tokenAddress The asset's contract address.
     * @param options.tokenId The asset's token ID, or null if ERC-20
     * @param retries Number of times to retry if the service is unavailable for any reason
     * @returns The {@link OpenSeaAsset} returned by the API.
     * @throws An error if the function is called on an unsupported chain.
     */
    getAsset({ tokenAddress, tokenId, }, retries = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            if (![types_1.Chain.Mainnet, types_1.Chain.Goerli].includes(this.chain)) {
                throw new Error("Please use `getNFT()` for multichain capabilities.");
            }
            let json;
            try {
                json = yield this.get(`${constants_1.API_V1_PATH}/asset/${tokenAddress}/${tokenId !== null && tokenId !== void 0 ? tokenId : 0}/`);
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(1000);
                return this.getAsset({ tokenAddress, tokenId }, retries - 1);
            }
            return (0, utils_2.assetFromJSON)(json);
        });
    }
    /**
     * Fetch multiple NFTs for a collection.
     * @param slug The slug (identifier) of the collection
     * @param limit The number of NFTs to retrieve. Must be greater than 0 and less than 51.
     * @param next Cursor to retrieve the next page of NFTs
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link ListNFTsResponse} returned by the API.
     */
    getNFTsByCollection(slug, limit = undefined, next = undefined, retries = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            let response;
            try {
                response = yield this.get((0, utils_1.getListNFTsByCollectionPath)(slug), {
                    limit,
                    next,
                });
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(1000);
                return this.getNFTsByCollection(slug, limit, next, retries - 1);
            }
            return response;
        });
    }
    /**
     * Fetch multiple NFTs for a contract.
     * @param chain The NFT's chain.
     * @param address The NFT's contract address.
     * @param limit The number of NFTs to retrieve. Must be greater than 0 and less than 51.
     * @param next Cursor to retrieve the next page of NFTs.
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The {@link ListNFTsResponse} returned by the API.
     */
    getNFTsByContract(chain, address, limit = undefined, next = undefined, retries = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            let response;
            try {
                response = yield this.get((0, utils_1.getListNFTsByContractPath)(chain, address), {
                    limit,
                    next,
                });
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(1000);
                return this.getNFTsByContract(chain, address, limit, next, retries - 1);
            }
            return response;
        });
    }
    /**
     * Fetch NFTs owned by an account.
     * @param address The address of the account
     * @param limit The number of NFTs to retrieve. Must be greater than 0 and less than 51.
     * @param next Cursor to retrieve the next page of NFTs
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @param chain The chain to query. Defaults to the chain set in the constructor.
     * @returns The {@link ListNFTsResponse} returned by the API.
     */
    getNFTsByAccount(address, limit = undefined, next = undefined, retries = 1, chain = this.chain) {
        return __awaiter(this, void 0, void 0, function* () {
            let response;
            try {
                response = yield this.get((0, utils_1.getListNFTsByAccountPath)(chain, address), {
                    limit,
                    next,
                });
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(1000);
                return this.getNFTsByAccount(address, limit, next, retries - 1, chain);
            }
            return response;
        });
    }
    /**
     * Fetch metadata, traits, ownership information, and rarity for a single NFT.
     * @param chain The NFT's chain.
     * @param address The NFT's contract address.
     * @param identifier the identifier of the NFT (i.e. Token ID)
     * @param retries Number of times to retry if the service is unavailable for any reason
     * @returns The {@link GetNFTResponse} returned by the API.
     */
    getNFT(chain, address, identifier, retries = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            let response;
            try {
                response = yield this.get((0, utils_1.getNFTPath)(chain, address, identifier));
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(1000);
                return this.getNFT(chain, address, identifier, retries - 1);
            }
            return response;
        });
    }
    /**
     * Fetch a list of assets.
     * @deprecated Use {@link getNFTsByContract} or {@link getNFTsByCollection} for multichain capabilities.
     * @param query Options to filter the list returned.
     * @param query.owner The wallet address of the owner of the assets.
     * @param query.asset_contract_address The Asset's contract address.
     * @param query.token_ids String array of token IDs to filter by.
     * @param query.order_by The field to order the list by.
     * @param query.order_direction The direction to order the list.
     * @param query.limit The number of assets to retrieve. Must be greater than 0 and less than 201.
     * @param query.cursor Cursor to retrieve the next page of assets.
     * @throws An error if the function is called on an unsupported chain.
     * @returns The {@link GetAssetsResponse} returned by the API.
     */
    getAssets(query = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            if (![types_1.Chain.Mainnet, types_1.Chain.Goerli].includes(this.chain)) {
                throw new Error("Please use `getNFTsByContract()` or `getNFTsByCollection()` for multichain capabilities.");
            }
            const json = yield this.get(`${constants_1.API_V1_PATH}/assets/`, Object.assign({ limit: this.pageSize }, query));
            return {
                assets: json.assets.map((j) => (0, utils_2.assetFromJSON)(j)),
                next: json.next,
                previous: json.previous,
                estimatedCount: json.estimated_count,
            };
        });
    }
    /**
     * Fetch an OpenSea collection.
     * @param slug The slug (identifier) of the collection.
     * @returns The {@link OpenSeaCollection} returned by the API.
     */
    getCollection(slug) {
        return __awaiter(this, void 0, void 0, function* () {
            const path = (0, utils_1.getCollectionPath)(slug);
            const response = yield this.get(path);
            return (0, utils_2.collectionFromJSON)(response.collection);
        });
    }
    /**
     * Fetch list of fungible tokens.
     * @param query Query to use for getting tokens. See {@link OpenSeaFungibleTokenQuery}.
     * @param page Page number to fetch. Defaults to 1.
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @throws An error if the function is called on an unsupported chain.
     * @returns The {@link GetPaymentTokensResponse} returned by the API.
     */
    getPaymentTokens(query = {}, page = 1, retries = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            if (![types_1.Chain.Mainnet, types_1.Chain.Goerli].includes(this.chain)) {
                throw new Error("This method does not work outside of Mainnet and Goerli chains as it uses the v1 API.");
            }
            let json;
            try {
                json = yield this.get(`${constants_1.API_V1_PATH}/tokens/`, Object.assign(Object.assign({}, query), { limit: this.pageSize, offset: (page - 1) * this.pageSize }));
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(1000);
                return this.getPaymentTokens(query, page, retries - 1);
            }
            return {
                tokens: json.map((t) => (0, utils_2.tokenFromJSON)(t)),
            };
        });
    }
    /**
     * Fetch a bundle from the API.
     * @param options
     * @param options.slug The bundle's identifier
     * @returns The {@link OpenSeaAssetBundle} returned by the API. If not found, returns null.
     */
    getBundle({ slug, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const json = yield this.get(`${constants_1.API_V1_PATH}/bundle/${slug}/`);
            return json ? (0, utils_2.assetBundleFromJSON)(json) : null;
        });
    }
    /**
     * Fetch list of bundles from the API.
     * @param query Query to use for getting bunldes. See {@link OpenSeaAssetBundleQuery}.
     * @param page Page number to fetch. Defaults to 1.
     * @returns The {@link GetBundlesResponse} returned by the API.
     */
    getBundles(query = {}, page = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            const json = yield this.get(`${constants_1.API_V1_PATH}/bundles/`, Object.assign(Object.assign({}, query), { limit: this.pageSize, offset: (page - 1) * this.pageSize }));
            return {
                bundles: json.bundles.map((j) => (0, utils_2.assetBundleFromJSON)(j)),
                estimatedCount: json.estimated_count,
            };
        });
    }
    /**
     * Force refresh the metadata for an NFT.
     * @param chain The chain where the NFT is located.
     * @param address The address of the NFT's contract.
     * @param identifier The identifier of the NFT.
     * @param retries Number of times to retry if the service is unavailable for any reason.
     * @returns The response from the API.
     */
    refreshNFTMetadata(chain, address, identifier, retries = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            let response;
            try {
                response = yield this.post((0, utils_1.getRefreshMetadataPath)(chain, address, identifier), {});
            }
            catch (error) {
                _throwOrContinue(error, retries);
                yield (0, utils_2.delay)(1000);
                return this.refreshNFTMetadata(chain, address, identifier, retries - 1);
            }
            return response;
        });
    }
    /**
     * Generic fetch method for any API endpoint
     * @param apiPath Path to URL endpoint under API
     * @param query URL query params. Will be used to create a URLSearchParams object.
     * @returns @typeParam T The response from the API.
     */
    get(apiPath, query = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const qs = this.objectToSearchParams(query);
            const url = `${this.apiBaseUrl}${apiPath}?${qs}`;
            return yield this._fetch({ url });
        });
    }
    /**
     * Generic post methd for any API endpoint.
     * @param apiPath Path to URL endpoint under API
     * @param body Data to send.
     * @param opts ethers ConnectionInfo, similar to Fetch API.
     * @returns @typeParam T The response from the API.
     */
    post(apiPath, body, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            const options = Object.assign({ url: `${this.apiBaseUrl}${apiPath}` }, opts);
            return yield this._fetch(options, body);
        });
    }
    objectToSearchParams(params = {}) {
        const urlSearchParams = new URLSearchParams();
        Object.entries(params).forEach(([key, value]) => {
            if (value && Array.isArray(value)) {
                value.forEach((item) => item && urlSearchParams.append(key, item));
            }
            else if (value) {
                urlSearchParams.append(key, value);
            }
        });
        return urlSearchParams.toString();
    }
    /**
     * Get from an API Endpoint, sending auth token in headers
     * @param opts ethers ConnectionInfo, similar to Fetch API
     * @param body Optional body to send. If set, will POST, otherwise GET
     */
    _fetch(opts, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const headers = Object.assign(Object.assign({ "x-app-id": "opensea-js" }, (this.apiKey ? { "X-API-KEY": this.apiKey } : {})), opts.headers);
            const req = Object.assign(Object.assign({}, opts), { headers });
            this.logger(`Sending request: ${opts.url} ${JSON.stringify(req).slice(0, 200)}...`);
            return yield ethers_1.ethers.utils.fetchJson(req, body ? JSON.stringify(body) : undefined);
        });
    }
}
exports.OpenSeaAPI = OpenSeaAPI;
function _throwOrContinue(error, retries) {
    const isUnavailable = error instanceof Error &&
        !!error.message &&
        (error.message.includes("503") || error.message.includes("429"));
    if (retries <= 0 || !isUnavailable) {
        throw error;
    }
}
//# sourceMappingURL=api.js.map