import { Seaport } from "@opensea/seaport-js";
import { BigNumber, BigNumberish, Wallet, providers } from "ethers";
import { OpenSeaAPI } from "./api/api";
import { Offer } from "./api/types";
import { OrderV2 } from "./orders/types";
import { Asset, ComputedFees, EventData, EventType, Chain, OpenSeaAPIConfig, OpenSeaAsset } from "./types";
/**
 * The OpenSea SDK main class.
 * @category Main Classes
 */
export declare class OpenSeaSDK {
    /** Provider to use for transactions. */
    provider: providers.JsonRpcProvider;
    /** Seaport v1.5 client  @see {@link https://github.com/ProjectOpenSea/seaport}*/
    seaport_v1_5: Seaport;
    /** Logger function to use when debugging */
    logger: (arg: string) => void;
    /** API instance */
    readonly api: OpenSeaAPI;
    /** The configured chain */
    readonly chain: Chain;
    private _emitter;
    private _signerOrProvider;
    /**
     * Create a new instance of OpenSeaJS.
     * @param provider Provider to use for transactions. For example:
     *  `const provider = new ethers.providers.JsonRpcProvider('https://mainnet.infura.io')`
     * @param apiConfig configuration options, including `chain`
     * @param logger function that will be called with debugging
     * @param wallet ethers wallet for order posting
     *  information
     */
    constructor(provider: providers.JsonRpcProvider, apiConfig?: OpenSeaAPIConfig, logger?: (arg: string) => void, wallet?: Wallet);
    /**
     * Add a listener for events emitted by the SDK.
     * @param event The {@link EventType} to listen to.
     * @param listener A callback that will accept an object with {@link EventData}\
     * @param once Whether the listener should only be called once, or continue listening until removed.
     */
    addListener(event: EventType, listener: (data: EventData) => void, once?: boolean): void;
    /**
     * Remove an event listener by calling `.removeListener()` on an event and listener.
     * @param event The {@link EventType} to remove a listener for\
     * @param listener The listener to remove
     */
    removeListener(event: EventType, listener: (data: EventData) => void): void;
    /**
     * Remove all event listeners. This should be called when you're unmounting
     * a component that listens to events to make UI updates.
     * @param event Optional EventType to remove listeners for
     */
    removeAllListeners(event?: EventType): void;
    /**
     * Wrap ETH into WETH.
     * W-ETH is needed for placing buy orders (making offers).
     * @param options
     * @param options.amountInEth Amount of ether to wrap
     * @param options.accountAddress Address of the user's wallet containing the ether
     */
    wrapEth({ amountInEth, accountAddress, }: {
        amountInEth: BigNumberish;
        accountAddress: string;
    }): Promise<void>;
    /**
     * Unwrap WETH into ETH.
     * Emits the `UnwrapWeth` event when the transaction is prompted.
     * @param options
     * @param options.amountInEth How much WETH to unwrap
     * @param options.accountAddress Address of the user's wallet containing the WETH
     */
    unwrapWeth({ amountInEth, accountAddress, }: {
        amountInEth: BigNumberish;
        accountAddress: string;
    }): Promise<void>;
    private getAmountWithBasisPointsApplied;
    private getFees;
    private getNFTItems;
    /**
     * Create a buy order to make an offer on an asset.
     * @param options
     * @param options.asset The asset to trade
     * @param options.accountAddress Address of the wallet making the buy order
     * @param options.startAmount Value of the offer in units, not base units e.g. not wei, of the payment token (or WETH if no payment token address specified)
     * @param options.quantity The number of assets to bid for (if fungible or semi-fungible). Defaults to 1.
     * @param options.domain An optional domain to be hashed and included in the first four bytes of the random salt.
     * @param options.salt Arbitrary salt. If not passed in, a random salt will be generated with the first four bytes being the domain hash or empty.
     * @param options.expirationTime Expiration time for the order, in UTC seconds
     * @param options.paymentTokenAddress ERC20 address for the payment token in the order. If unspecified, defaults to WETH
     * @returns The {@link OrderV2} that was created.
     *
     * @throws Error if the asset does not contain a token id.
     * @throws Error if the accountAddress is not available through wallet or provider.
     * @throws Error if the startAmount is not greater than 0.
     * @throws Error if paymentTokenAddress is not WETH on anything other than Ethereum mainnet.
     */
    createBuyOrder({ asset, accountAddress, startAmount, quantity, domain, salt, expirationTime, paymentTokenAddress, }: {
        asset: Asset;
        accountAddress: string;
        startAmount: BigNumberish;
        quantity?: BigNumberish;
        domain?: string;
        salt?: BigNumberish;
        expirationTime?: BigNumberish;
        paymentTokenAddress?: string;
    }): Promise<OrderV2>;
    /**
     * Create a sell order to make a listing for a asset.
     * @param options
     * @param options.asset The asset to trade
     * @param options.accountAddress  Address of the wallet making the buy order
     * @param options.startAmount Value of the listing at the start of the auction in units, not base units e.g. not wei, of the payment token (or WETH if no payment token address specified)
     * @param options.endAmount Value of the listing at the end of the auction. If specified, price will change linearly between startAmount and endAmount as time progresses.
     * @param options.quantity The number of assets to list (if fungible or semi-fungible). Defaults to 1.
     * @param options.domain An optional domain to be hashed and included in the first four bytes of the random salt. This can be used for on-chain order attribution to assist with analytics.
     * @param options.salt Arbitrary salt. If not passed in, a random salt will be generated with the first four bytes being the domain hash or empty.
     * @param options.listingTime Optional time when the order will become fulfillable, in UTC seconds. Undefined means it will start now.
     * @param options.expirationTime Expiration time for the order, in UTC seconds.
     * @param options.paymentTokenAddress ERC20 address for the payment token in the order. If unspecified, defaults to ETH
     * @param options.buyerAddress Optional address that's allowed to purchase this item. If specified, no other address will be able to take the order, unless its value is the null address.
     * @param options.englishAuction If true, the order will be listed as an English auction.
     * @returns The {@link OrderV2} that was created.
     *
     * @throws Error if the asset does not contain a token id.
     * @throws Error if the accountAddress is not available through wallet or provider.
     * @throws Error if the startAmount is not greater than 0.
     * @throws Error if paymentTokenAddress is not WETH on anything other than Ethereum mainnet.
     */
    createSellOrder({ asset, accountAddress, startAmount, endAmount, quantity, domain, salt, listingTime, expirationTime, paymentTokenAddress, buyerAddress, englishAuction, }: {
        asset: Asset;
        accountAddress: string;
        startAmount: BigNumberish;
        endAmount?: BigNumberish;
        quantity?: BigNumberish;
        domain?: string;
        salt?: BigNumberish;
        listingTime?: number;
        expirationTime?: number;
        paymentTokenAddress?: string;
        buyerAddress?: string;
        englishAuction?: boolean;
    }): Promise<OrderV2>;
    /**
     * Create a offer (buy order) for a collection.
     * @param options
     * @param options.collectionSlug Identifier for the collection.
     * @param options.accountAddress Address of the wallet making the offer.
     * @param options.amount Value of the offer in units, not base units e.g. not wei, of the payment token (or WETH if no payment token address specified).
     * @param options.quantity The number of assets to bid for (if fungible or semi-fungible).
     * @param options.domain An optional domain to be hashed and included in the first four bytes of the random salt. This can be used for on-chain order attribution to assist with analytics.
     * @param options.salt Arbitrary salt. If not passed in, a random salt will be generated with the first four bytes being the domain hash or empty.
     * @param options.expirationTime Expiration time for the order, in UTC seconds.
     * @param options.paymentTokenAddress ERC20 address for the payment token in the order. If unspecified, defaults to WETH.
     * @returns The {@link Offer} that was created.
     */
    createCollectionOffer({ collectionSlug, accountAddress, amount, quantity, domain, salt, expirationTime, paymentTokenAddress, }: {
        collectionSlug: string;
        accountAddress: string;
        amount: BigNumberish;
        quantity: number;
        domain?: string;
        salt?: BigNumberish;
        expirationTime?: number | string;
        paymentTokenAddress: string;
    }): Promise<Offer | null>;
    private fulfillPrivateOrder;
    /**
     * Fulfill or "take" an order for an asset. The order can be either a buy or sell order.
     * @param options
     * @param options.order The order to fulfill, a.k.a. "take"
     * @param options.accountAddress Address of the wallet taking the offer.
     * @param options.recipientAddress The optional address to receive the order's item(s) or curriencies. If not specified, defaults to accountAddress.
     * @param options.domain An optional domain to be hashed and included at the end of fulfillment calldata.  This can be used for on-chain order attribution to assist with analytics.
     * @returns Transaction hash of the order.
     *
     * @throws Error if the accountAddress is not available through wallet or provider.
     * @throws Error if the order's protocol address is not supported by OpenSea. See {@link isValidProtocol}.
     * @throws Error if attempting to fulfill the order with a recipient address which does not match a private listing.
     */
    fulfillOrder({ order, accountAddress, recipientAddress, domain, }: {
        order: OrderV2;
        accountAddress: string;
        recipientAddress?: string;
        domain?: string;
    }): Promise<string>;
    private cancelSeaportOrders;
    /**
     * Cancel an order on-chain, preventing it from ever being fulfilled.
     * @param options
     * @param options.order The order to cancel
     * @param options.accountAddress The account address that will be cancelling the order.
     * @param options.domain An optional domain to be hashed and included at the end of fulfillment calldata.  This can be used for on-chain order attribution to assist with analytics.
     *
     * @throws Error if the accountAddress is not available through wallet or provider.
     * @throws Error if the order's protocol address is not supported by OpenSea. See {@link isValidProtocol}.
     */
    cancelOrder({ order, accountAddress, domain, }: {
        order: OrderV2;
        accountAddress: string;
        domain?: string;
    }): Promise<void>;
    /**
     * Returns whether an order is fulfillable.
     * An order may not be fulfillable if a target item's transfer function
     * is locked for some reason, e.g. an item is being rented within a game
     * or trading has been locked for an item type.
     * @param options
     * @param options.order Order to check
     * @param options.accountAddress The account address that will be fulfilling the order
     * @returns True if the order is fulfillable, else False.
     *
     * @throws Error if the order's protocol address is not supported by OpenSea. See {@link isValidProtocol}.
     */
    isOrderFulfillable({ order, accountAddress, }: {
        order: OrderV2;
        accountAddress: string;
    }): Promise<boolean>;
    /**
     * Get an account's balance of any Asset. This asset can be an ERC20, ERC1155, or ERC721.
     * @param options
     * @param options.accountAddress Account address to check
     * @param options.asset The Asset to check balance for
     * @param retries How many times to retry if balance is 0. Defaults to 1.
     * @returns The balance of the asset for the account.
     *
     * @throws Error if the token standard does not support balanceOf.
     */
    getBalance({ accountAddress, asset }: {
        accountAddress: string;
        asset: Asset;
    }, retries?: number): Promise<BigNumber>;
    /**
     * Compute the fees for an order.
     * @param options
     * @param options.asset Asset to use for fees. May be blank ONLY for multi-collection bundles.
     * @returns The {@link ComputedFees} for the order.
     */
    computeFees({ asset, }: {
        asset?: OpenSeaAsset;
    }): Promise<ComputedFees>;
    /**
     * Instead of signing an off-chain order, this methods allows you to approve an order
     * with on on-chain transaction.
     * @param order Order to approve
     * @param domain An optional domain to be hashed and included at the end of fulfillment calldata.  This can be used for on-chain order attribution to assist with analytics.
     * @returns Transaction hash of the approval transaction
     *
     * @throws Error if the accountAddress is not available through wallet or provider.
     * @throws Error if the order's protocol address is not supported by OpenSea. See {@link isValidProtocol}.
     */
    approveOrder(order: OrderV2, domain?: string): Promise<string>;
    /**
     * Compute the `basePrice` and `extra` parameters to be used to price an order.
     * Also validates the expiration time and auction type.
     * @param tokenAddress Address of the ERC-20 token to use for trading.
     * Use the null address for ETH
     * @param expirationTime When the auction expires, or 0 if never.
     * @param startAmount The base value for the order, in the token's main units (e.g. ETH instead of wei)
     * @param endAmount The end value for the order, in the token's main units (e.g. ETH instead of wei). If unspecified, the order's `extra` attribute will be 0
     */
    private _getPriceParameters;
    private _dispatch;
    /**
     * Throws an error if an account is not available through the provider.
     *
     * @param accountAddress The account address to check is available.
     */
    private _requireAccountIsAvailable;
    private _confirmTransaction;
}
