import { ItemType } from "@opensea/seaport-js/lib/constants";
import { BigNumber, ethers } from "ethers";
import { AssetEvent, Chain, OpenSeaAccount, OpenSeaAsset, OpenSeaAssetBundle, OpenSeaAssetContract, OpenSeaCollection, OpenSeaFungibleToken, OpenSeaUser, Order, TokenStandard, Transaction, UnsignedOrder } from "../types";
export declare const assetFromJSON: (asset: any) => OpenSeaAsset;
export declare const assetEventFromJSON: (assetEvent: any) => AssetEvent;
export declare const transactionFromJSON: (transaction: any) => Transaction;
export declare const accountFromJSON: (account: any) => OpenSeaAccount;
export declare const userFromJSON: (user: any) => OpenSeaUser;
export declare const assetBundleFromJSON: (asset_bundle: any) => OpenSeaAssetBundle;
export declare const assetContractFromJSON: (asset_contract: any) => OpenSeaAssetContract;
export declare const collectionFromJSON: (collection: any) => OpenSeaCollection;
export declare const tokenFromJSON: (token: any) => OpenSeaFungibleToken;
/**
 * Estimate gas usage for a transaction.
 * @param provider The Provider
 * @param from Address sending transaction
 * @param to Destination contract address
 * @param data Data to send to contract
 * @param value Value in ETH to send with data
 */
export declare function estimateGas(provider: ethers.providers.Provider, { from, to, data, value }: ethers.Transaction): Promise<BigNumber>;
/**
 * Assign an order and a new matching order to their buy/sell sides
 * @param order Original order
 * @param matchingOrder The result of _makeMatchingOrder
 */
export declare function assignOrdersToSides(order: Order, matchingOrder: UnsignedOrder): {
    buy: Order;
    sell: Order;
};
/**
 * Delay using setTimeout
 * @param ms milliseconds to wait
 */
export declare function delay(ms: number): Promise<unknown>;
/**
 * Get special-case approval addresses for an erc721 contract
 * @param erc721Contract contract to check
 */
export declare function getNonCompliantApprovalAddress(erc721Contract: ethers.Contract, tokenId: string, _accountAddress: string): Promise<string | undefined>;
/**
 * The longest time that an order is valid for is one month from the current date
 * @returns unix timestamp
 */
export declare const getMaxOrderExpirationTimestamp: () => number;
interface ErrorWithCode extends Error {
    code: string;
}
export declare const hasErrorCode: (error: unknown) => error is ErrorWithCode;
export declare const getAssetItemType: (tokenStandard?: TokenStandard) => ItemType.ERC20 | ItemType.ERC721 | ItemType.ERC1155;
export declare const getWETHAddress: (chain: Chain) => "0xc02aaa39b223fe8d0a0e5c4f27ead9083c756cc2" | "0x7ceB23fD6bC0adD59E62ac25578270cFf1b9f619" | "0xb4fbf271143f4fbf7b91a5ded31805e42b2208d6" | "0xfFf9976782d46CC05630D1f6eBAb18b2324d6B14" | "0xfd844c2fca5e595004b17615f891620d1cb9bbb2" | "0x9330dd6713c8328a8d82b14e3f60a0f0b4cc7bfb" | "0xB31f66AA3C1e785363F0875A1B74E27b85FD66c7" | "0xd00ae08403B9bbb9124bB305C09058E32C39A48c" | "0xbb4CdB9CBd36B01bD1cBaEBF2De08d9173bc095c" | "0xae13d989dac2f0debff460ac112a837c89baa7cd" | "0x4200000000000000000000000000000000000006";
/**
 * Checks if the token address is the shared storefront address and if so replaces
 * that address with the lazy mint adapter addres. Otherwise, returns the input token address
 * @param tokenAddress token address
 * @returns input token address or lazy mint adapter address
 */
export declare const getAddressAfterRemappingSharedStorefrontAddressToLazyMintAdapterAddress: (tokenAddress: string) => string;
/**
 * Sums up the basis points for an Opensea or seller fee map and returns the
 * single numeric value if the map is not empty. Otherwise, it returns 0
 * @param fees a `Fees` submap holding fees (either Fees.openseaFees
 *  or Fees.sellerFees)
 * @returns sum of basis points in a fee map
 */
export declare const feesToBasisPoints: (fees: Map<string, number> | undefined) => number;
/**
 * Checks whether the current chain is a test chain.
 * @param chain Chain to check.
 * @returns True if the chain is a test chain.
 */
export declare const isTestChain: (chain: Chain) => boolean;
/**
 * Returns if a protocol address is valid.
 * @param protocolAddress The protocol address
 */
export declare const isValidProtocol: (protocolAddress: string) => boolean;
/**
 * Throws an error if the protocol address is not valid.
 * @param protocolAddress The protocol address
 */
export declare const requireValidProtocol: (protocolAddress: string) => void;
/**
 * Decodes an encoded string of token IDs into an array of individual token IDs using BigNumber for precise calculations.
 *
 * The encoded token IDs can be in the following formats:
 * 1. Single numbers: '123' => ['123']
 * 2. Comma-separated numbers: '1,2,3,4' => ['1', '2', '3', '4']
 * 3. Ranges of numbers: '5:8' => ['5', '6', '7', '8']
 * 4. Combinations of single numbers and ranges: '1,3:5,8' => ['1', '3', '4', '5', '8']
 * 5. Wildcard '*' (matches all token IDs): '*' => ['*']
 *
 * @param encodedTokenIds - The encoded string of token IDs to be decoded.
 * @returns An array of individual token IDs after decoding the input.
 *
 * @throws {Error} If the input is not correctly formatted or if BigNumber operations fail.
 *
 * @example
 * const encoded = '1,3:5,8';
 * const decoded = decodeTokenIds(encoded); // Output: ['1', '3', '4', '5', '8']
 *
 * @example
 * const encodedWildcard = '*';
 * const decodedWildcard = decodeTokenIds(encodedWildcard); // Output: ['*']
 *
 * @example
 * const emptyEncoded = '';
 * const decodedEmpty = decodeTokenIds(emptyEncoded); // Output: []
 */
export declare const decodeTokenIds: (encodedTokenIds: string) => string[];
export {};
